@@ -36,6 +36,13 @@ import time
 import random
 import threading
 
+from xml.sax import make_parser
+from xml.sax import saxutils
+from xml.sax import handler
+from xml.sax.handler import feature_namespaces
+
+VERSION = "1.1.7"
+
 try:
     import hildon
 except:
@@ -43,10 +50,12 @@ except:
 else:
     hasHildon = True
 
-from xml.sax import make_parser
-from xml.sax import saxutils
-from xml.sax import handler
-from xml.sax.handler import feature_namespaces
+try :
+    # Nasty hack to see if we're running on a Neo Freerunner
+    fd = os.open( "/sys/bus/platform/devices/neo1973-pm-bt.0/power_on", os.O_RDWR )
+    isNeo = True
+except :
+    isNeo = False
 
 class ReadPyringXML( handler.ContentHandler ) :
 
@@ -131,13 +140,22 @@ class ReadPyringXML( handler.ContentHandler ) :
 
 class Main:
     def __init__(self):
-        # do some initalization  
+        # do some initalization
+        print "Pyring verison", VERSION
+        print "copyright 2008-2009 Angus Ainslie <angus@handheldshell.com>"
+
         if hasHildon :
             print "Using hildon UI"
+        elif isNeo :
+            print "Using Neo UI"
         else :
-            print "Using Standard UI"
+            print "Using standard UI"
+
+        if isNeo :
+            self.gladeFile = "/usr/lib/pyring/neo_gui.glade"
+        else :
+            self.gladeFile = "/usr/lib/pyring/gui.glade"
 
-        self.gladeFile = "/usr/lib/pyring/gui.glade"
         self.wTree = gtk.glade.XML( self.gladeFile, "mainwindow" )
 
         signals = { 
@@ -160,6 +178,7 @@ class Main:
             "on_account_text_changed" : self.OnAccountTextChanged,
             "on_password_text_changed" : self.OnPasswordTextChanged,
             "on_note_text_changed" : self.OnNoteTextChanged,
+            "on_main_genpwd_btn_clicked" : self.OnMainGenpwdBtn,
             "on_main_update_btn_activate" : self.OnMainUpdateBtn,
             "on_main_cancel_btn_activate" : self.OnMainCancelBtn,
         }
@@ -208,7 +227,7 @@ class Main:
         self.nameView = self.wTree.get_widget("nameView")
         
         self.AddListColumn(self.sName, self.cName, True)
-        self.AddListColumn(self.sUsername, self.cUsername, False)
+#        self.AddListColumn(self.sUsername, self.cUsername, False)
 
         self.nameList = gtk.ListStore(str, str)
         self.nameList.connect("sort-column-changed", self.RecordSort)
@@ -374,8 +393,8 @@ class Main:
                 return False
 
     def AlphaOnly( self, text ) :
-        """list views have a funny way of sorting on on the alpha chars in the 
-        list - This tries to do the same thing"""
+        """list views have a funny way of sorting on only the alpha
+        chars in the list - This tries to do the same thing"""
         ret = ''
         a = []
 
@@ -672,12 +691,36 @@ class Main:
         updateBtn.set_sensitive( False )
         cancelBtn = self.wTree.get_widget("main_cancel_btn")
         cancelBtn.set_sensitive( False )
+        pwdBtn = self.wTree.get_widget("main_genpwd_btn")
+        pwdBtn.set_sensitive( False )
 
     def ActivateButtons( self ) :
         updateBtn = self.wTree.get_widget("main_update_btn")
         updateBtn.set_sensitive( True )
         cancelBtn = self.wTree.get_widget("main_cancel_btn")
         cancelBtn.set_sensitive( True )
+        pwdBtn = self.wTree.get_widget("main_genpwd_btn")
+        pwdBtn.set_sensitive( True )
+
+    def DeactivateFields( self ) :
+        fld = self.wTree.get_widget("name_text")
+        fld.set_sensitive( False )
+        fld = self.wTree.get_widget("account_text")
+        fld.set_sensitive( False )
+        fld = self.wTree.get_widget("password_text")
+        fld.set_sensitive( False )
+        fld = self.wTree.get_widget("note_text")
+        fld.set_sensitive( False )
+
+    def ActivateFields( self ) :
+        fld = self.wTree.get_widget("name_text")
+        fld.set_sensitive( True )
+        fld = self.wTree.get_widget("account_text")
+        fld.set_sensitive( True )
+        fld = self.wTree.get_widget("password_text")
+        fld.set_sensitive( True )
+        fld = self.wTree.get_widget("note_text")
+        fld.set_sensitive( True )
 
     def SetFields( self, name, account, password, note ) :
         if name != None :
@@ -725,6 +768,9 @@ class Main:
 
         self.ClearButtons()
         self.ClearFields()
+        self.ActivateFields()
+        pwdBtn = self.wTree.get_widget("main_genpwd_btn")
+        pwdBtn.set_sensitive( True )
         self.recordChanged = 1
 
     def OnRowDelete( self, one ) :
@@ -753,6 +799,54 @@ class Main:
                 self.ClearFields()
                 self.recordChanged = 0
 
+    def OnMainGenpwdBtn(self, event):
+        genpwdTree = gtk.glade.XML(self.gladeFile, "genpwd_dlg")
+        self.genpwdDlg = genpwdTree.get_widget("genpwd_dlg")
+
+        if self.genpwdDlg.run() == gtk.RESPONSE_OK:
+	    pwdlen = self.GetPwdLen(genpwdTree)
+	    pwdchars = self.GetPwdChars(genpwdTree)
+            self.Genpwd(pwdlen, pwdchars)
+        self.genpwdDlg.destroy()
+
+    def GetPwdLen (self, tree):
+	if tree.get_widget("pwdlen_4").get_active():
+	    return 4
+	elif tree.get_widget("pwdlen_6").get_active():
+	    return 6
+	elif tree.get_widget("pwdlen_8").get_active():
+	    return 8
+	elif tree.get_widget("pwdlen_10").get_active():
+	    return 10
+	elif tree.get_widget("pwdlen_16").get_active():
+	    return 16
+	elif tree.get_widget("pwdlen_20").get_active():
+	    return 20
+
+    def GetPwdChars (self, tree):
+	chars = ''
+	if tree.get_widget("pwdinc_lowercase").get_active():
+	    chars += string.lowercase
+	if tree.get_widget("pwdinc_uppercase").get_active():
+	    chars += string.uppercase
+	if tree.get_widget("pwdinc_digits").get_active():
+	    chars += string.digits
+	if tree.get_widget("pwdinc_punctuation").get_active():
+	    chars += string.punctuation
+	return chars
+
+    def Genpwd(self, length, chars):
+	"""Generate a password of length characters, using characters
+	in chars."""
+        tree = self.wTree
+        fld = tree.get_widget('password_text')
+	if chars == '':
+	    return
+	# Create a new seed every time.  Keyring uses 256 bytes
+	random.seed(os.urandom(256))
+	password = ''.join([random.choice(chars) for i in range(length)])
+	fld.set_text(password)
+
     def OnMainUpdateBtn( self, event ) :
         self.ClearButtons()
 
@@ -785,9 +879,9 @@ class Main:
 
     def OnRowActivated( self, tree, event ) :
         cur = self.nameView.get_cursor()
-        if cur == None :
+        if cur == None or cur[0] == None :
             return
-        
+
         self.DisplayRecord( cur[0][0] )
 
     def DisplayRecord( self, recordNum ) :
@@ -809,6 +903,7 @@ class Main:
                             record[1], record[2] )
         self.recordChanged = 0
         self.ClearButtons()
+        self.ActivateFields()
         self.curRecord = recordNum
         
     def OnSelectRow( self, editing, user_param ) :
